#include <bsp.h>
#include <stdio.h>
#include <sys/types.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <rtems/libio.h>

rtems_task chroot_task(
  rtems_task_argument argument
);


rtems_task Init(
  rtems_task_argument ignored
)
{
    rtems_id task_ID;
    rtems_status_code status_code;
    rtems_task_entry task_entry = chroot_task;
    
    /* Create & start task */
    status_code = rtems_task_create(
        rtems_build_name('T', 'S', 'K', ' '),
        10, 
        10*RTEMS_MINIMUM_STACK_SIZE,
        RTEMS_DEFAULT_MODES,
        RTEMS_DEFAULT_ATTRIBUTES,
        &task_ID);
    if (status_code != RTEMS_SUCCESSFUL)
    {
        puts("Can't create task\n");
        exit(0);
    }
    status_code = rtems_task_start(task_ID, task_entry, 0);
    if (status_code != RTEMS_SUCCESSFUL)
    {
        puts("Can't start task\n");
        exit(0);
    }
    status_code = rtems_task_wake_after(RTEMS_YIELD_PROCESSOR);
    if (status_code != RTEMS_SUCCESSFUL)
    {
        puts("Can't yield processor\n");
        exit(0);
    }
    status_code = rtems_task_delete(task_ID);
    if (status_code != RTEMS_SUCCESSFUL)
    {
        puts("chroot_task: Can't delete myself\n");
        exit(0);
    }
    puts("SUCCESS\n");
    exit(0);
}

rtems_task chroot_task(rtems_task_argument argument)
{
    rtems_status_code status_code;
    if (chroot("/dev") != 0)
    {
        perror("chroot");
        exit(0);
    }
    status_code = rtems_task_suspend(RTEMS_SELF);
    if (status_code != RTEMS_SUCCESSFUL)
    {
        puts("chroot_task: Can't suspend myself\n");
        exit(0);
    }
}

#define CONFIGURE_INIT

#define CONFIGURE_MAXIMUM_TASKS                        2
#define CONFIGURE_INIT_TASK_PRIORITY                   50

#define CONFIGURE_RTEMS_INIT_TASKS_TABLE

#define CONFIGURE_INIT_TASK_STACK_SIZE         4*RTEMS_MINIMUM_STACK_SIZE

#define CONFIGURE_TEST_NEEDS_CONSOLE_DRIVER
#define CONFIGURE_APPLICATION_NEEDS_CLOCK_DRIVER

#define CONFIGURE_LIBIO_MAXIMUM_FILE_DESCRIPTORS       5
#define CONFIGURE_USE_IMFS_AS_BASE_FILESYSTEM

#define STACK_CHECKER_ON

#include <confdefs.h>
